"""\
<b>EpsUtil - Graphical Epson Ink Jet Printer Toolbox</b>

<p>This program is based on the command line utility escputil 
by Robert Krawitz, available from the gimp-print package 
[http://sf.net/projects/gimp-print/].</p>

<p>It is able to perform the basic maintainance steps: check ink
level, test nozzles, and clean heads. I refrained from supporting 
the inherently dangerous head alignment options by now.</p>

<p>Copyright 2002-2003 Hans-Peter Jansen &lt;hpj@urpla.net&gt;</p>

<p>Internals</p>

<p>The UI was created with Qt Designer [http://trolltech.com],
eric3 [http://www.die-offenbachs.de/detlev/eric3.html], 
PyQt [http://www.riverbankcomputing.co.uk/], and Python 
[http://python.org], which form a great development combo, btw.</p>

<p>This program is placed under the GNU General Public License V.2
(included in the package)</p>

<p>Contributions:</p>
<ul>
 <li>2003-04-03  Joe Curto:<br>
                 started italian translation
 <li>2003-06-25  Konstantin Korikov:<br>
                 better handling of old models and Qt styles
</ul>

<p>Changelog:</p>
<ul>
 <li>2002-07-16  started
 <li>2002-07-22  released as v0.1
 <li>2003-08-19  fixed locale problem with escputil<br>
                 prepared for translations, about box<br>
                 included contributions<br>
                 released as v0.2-rc1<br>
 <li>2003-08-26  Makefile cleanup/missing about.py target<br>
                 debug rudiment from escputil detection removed<br>
                 released as v0.2-rc2<br>
 <li>2003-08-30  epsutil.py refactored<br>
                 escputil timeout added (2 sec)<br>
                 error handling reworked<br>
                 ^Q shortcut and application icon added<br>
                 released as v0.2-rc3<br>
</ul>

<p>Enjoy,<br>
Pete</p>
"""
version = '0.2-rc3'

import sys
import os
import re
import string
import signal
import popen2

import config

from qt import *

from epsdial import epsDial
from about import aboutDial


styleDict = {}
try:    styleDict["platinum"] = QPlatinumStyle
except: pass
try:    styleDict["windows"] = QWindowsStyle
except: pass
try:    styleDict["motif"] = QMotifStyle
except: pass
try:    styleDict["motif+"] = QMotifPlusStyle
except: pass
try:    styleDict["cde"] = QCDEStyle
except: pass


oldPrinterModels = [        
     'C20sx', 'C20ux', 'C60', 'C70', 'C80',
     'color','pro', 'pro-xl',
     '400', '440', '460', '480', '500', '580', 
     '600', '640', '660', '670', '680' ]


class epsUtil:
    def __init__(self):
        self._epsutil = "escputil"
        self._escpVersion = None
        self._pipe = None
        # _modelDict dict key: real model, value: model id
        self._modelDict = {}
        self._escpver = re.compile(r"Escputil version (\d+(\.\d*)+)")
        self._pmre = re.compile(r"^\s+(\w+)\s+(.*)$")
        self._imodel = re.compile(r"MFG:(.*);CMD:(.*);MDL:(.*);CLS:(.*);DES:(.*);")
        self._clheads = re.compile(r"^Cleaning heads")
        self._rncheck = re.compile(r"^Running nozzle check")
        self._ilblack = re.compile(r"^\s+Black\s+(\w+)")
        self._ilcyan =  re.compile(r"^\s+Cyan\s+(\w+)")
        self._ilmagenta = re.compile(r"^\s+Magenta\s+(\w+)")
        self._ilyellow = re.compile(r"^\s+Yellow\s+(\w+)")
        self._ilcyanli = re.compile(r"^\s+Light Cyan\s+(\w+)")
        self._ilmagentali = re.compile(r"^\s+Light Magenta\s+(\w+)")

        out, err = self._call(self._epsutil)
        if out:
            m = self._escpver.search(out[0])
            if m:
                self._escpVersion = m.group(1)
        if not self._escpVersion:
            raise RuntimeError, err


    def escpVersion(self):
        return self._escpVersion


    def modelDict(self):
        if not self._modelDict:
            out, err = self._call(self._epsutil, "-M")
            for l in out:
                m = self._pmre.search(l)
                if m:
                    self._modelDict[m.group(2)] = m.group(1)
        return self._modelDict


    def modelOpt(self, model):
        ret = ''
        if not self._modelDict[model] in oldPrinterModels:
            ret = 'u'
        return ret


    def inquire(self, device):
        mdl = ''
        out, err = self._call(self._epsutil, "-dr %s" % device)
        for l in out:
            m = self._imodel.search(l)
            if m:
                mdl = m.group(3)
                break
        return mdl, err


    def cleanHead(self, device, model):
        ret = 0
        out, err = self._call(self._epsutil, "-q%scr %s" %
                                (self.modelOpt(model), device))
        for l in out:
            m = self._clheads.search(l)
            if m:
                ret = 1
        return ret, err


    def nozzleTest(self, device, model):
        ret = 0
        out, err = self._call(self._epsutil, "-q%snr %s" %
                                (self.modelOpt(model), device))
        for l in out:
            m = self._rncheck.search(l)
            if m:
                ret = 1
        return ret, err


    def inkLevel(self, device, model):
        il = [-1, -1, -1, -1, -1, -1]
        out, err = self._call(self._epsutil, "-q%sir %s" %
                                (self.modelOpt(model), device))
        rx = (self._ilblack, self._ilcyan, self._ilmagenta,
              self._ilyellow, self._ilcyanli, self._ilmagentali)
        for l in out:
            for i in range(6):
                m = rx[i].search(l)
                if m:
                    il[i] = int(m.group(1))
                    break
        return il, err


    def _call(self, tool, arg = ""):
        """ read from pipe
            return (stdout, stderr) tuple with list of lines
        """
        out, err = [], []

        def _cancel(signum, frame):
            if self._pipe:
                os.kill(self._pipe.pid, signal.SIGHUP)
                err.append(qApp.translate("epsUtil", "killed due to timeout condition\n"))

        signal.signal(signal.SIGALRM, _cancel)
        signal.alarm(2)
        #print '%s %s' % (tool, arg)
        self._pipe = popen2.Popen3('LANG=C %s %s' % (tool, arg), 1)
        try:
            while 1:
                l = self._pipe.fromchild.readline()
                if not l:
                    break
                out.append(l)
            while 1:
                l = self._pipe.childerr.readline()
                if not l:
                    break
                err.append(l)
        except:
            pass
        signal.alarm(0)
        self._pipe = None
        #print out, err
        if err:
            # return first stderr line without NL
            err = err[0][:-1]
        else:
            err = ""
        return out, err


class mainWindow(epsDial):
    def __init__(self, app, parent = None, name = None, fl = 0):
        self.app = app
        self.model = None
        self.cfg = config.Config()
        self.cfg.load()
        if styleDict.has_key(self.cfg.style):
            app.setStyle(styleDict[self.cfg.style]())
        # set up parent
        epsDial.__init__(self, parent, name, fl)
        wh = self.cfg.mainScreenSize
        if wh:
            self.resize(wh[0], wh[1])
        xy = self.cfg.mainScreenLocation
        if xy:
            self.move(xy[0], xy[1])


    def accept(self):
        self.closeEvent()
        QDialog.accept(self)


    def closeEvent(self, e = None):
        # save location & size
        r = self.geometry()
        if self.cfg.saveLayout:
            self.cfg.mainScreenSize = [r.width(), r.height()]
        r = self.frameGeometry()
        if self.cfg.saveLocation:
            self.cfg.mainScreenLocation = [r.x(), r.y()]
        # save devices
        self.cfg.devices = []
        for i in range(self.cbDevice.count()):
            self.cfg.devices.append(str(self.cbDevice.text(i)))
        self.cfg.devicesIdx = self.cbDevice.currentItem()
        # save model (if not first)
        if self.cbModel.currentItem():
            self.cfg.model = str(self.cbModel.currentText())
        #print "save config..."
        self.cfg.save()
        sys.exit(0)


    def polish(self):
        if not self.__dict__.has_key("epsutil"):
            try:
                self.epsutil = epsUtil()
            except RuntimeError, msg:
                QMessageBox.warning(self, "EpsUtil",
                    self.trUtf8("Command line utility <b>escputil</b> not found.<br><br>"
                    "Please make sure, this utility from the <b>gimp-print</b> project is "
                    "correctly installed.<br><br>The <b>gimp-print</b> project is located at "
                    "http://sourceforge.net/projects/gimp-print/.<br>"),
                    QMessageBox.Abort | QMessageBox.Escape)
                sys.exit(0)
            # setup model
            self.cbModel.insertItem(self.trUtf8("Choose Model"))
            pm = self.epsutil.modelDict().keys()
            pm.sort()
            self.cbModel.insertStrList(pm)
            m = self.cfg.model
            if m and m in pm:
                self.cbModel.setCurrentText(m)
                self.model = m
            else:
                self.enableControls(0)

            # setup device
            map(self.cbDevice.insertItem, self.cfg.devices)
            self.cbDevice.setCurrentItem(self.cfg.devicesIdx)

            # ^q shortcut
            a = QAccel(self)
            a.connectItem(a.insertItem(QKeySequence("Ctrl+Q")), self.closeEvent)

            # signals & slots
            self.connect(self.cbModel, SIGNAL("activated(int)"), self.modelSelected)
            self.connect(self.pbCleanHead, SIGNAL("clicked()"), self.cleanHead)
            self.connect(self.pbNozzleTest, SIGNAL("clicked()"), self.nozzleTest)
            self.connect(self.pbInkLevel, SIGNAL("clicked()"), self.inkLevel)
            self.connect(self.pbAbout, SIGNAL("clicked()"), self.about)

            # about dialog
            self.ad = aboutDial(self)
            self.ad.adTitle.setText("<h2><b>EpsUtil v%s</b></h2>" % (version))
            # gather involved versions
            pyver = string.split(sys.version, ' ')[0]
            qtver, pyqtver = ("?", "?")
            try:    qtver = qVersion()
            except: pass
            try:    pyqtver = PYQT_VERSION_STR
            except: pass
            self.ad.adSystem.setText("Escputil: %s  PyQt: %s  Qt: %s  Python: %s" % (
                                     self.epsutil.escpVersion(), pyqtver, qtver, pyver))
            self.ad.adText.setText(__doc__)
            self.connect(self.ad.adClose, SIGNAL("clicked()"), self.aboutDone)
        # let the base class know, we're ready now
        QDialog.polish(self)


    def about(self):
        self.ad.show()


    def aboutDone(self):
        self.ad.close()


    def modelSelected(self, idx):
        self.enableControls(not idx == 0)
        if idx:
            self.model = str(self.cbModel.currentText())
        else:
            self.model = None


    def enableControls(self, enable):
        self.pbCleanHead.setEnabled(enable)
        self.pbNozzleTest.setEnabled(enable)
        self.pbInkLevel.setEnabled(enable)


    def cleanHead(self):
        ok, err = self.epsutil.cleanHead(str(self.cbDevice.currentText()), self.model)
        if not ok:
            self.errMessage(err)


    def nozzleTest(self):
        ok, err = self.epsutil.nozzleTest(str(self.cbDevice.currentText()), self.model)
        if not ok:
            self.errMessage(err)


    def inkLevel(self):
        bg = self.eraseColor()
        dev = self.cbDevice.currentText()
        il, err = self.epsutil.inkLevel(str(dev), self.model)
        if not err:
            # set ink levels
            i = 0
            for fr, pb, col in (
                    (self.frBlack, self.ilBlack, (0, 0, 0)),
                    (self.frCyan, self.ilCyan, (0, 255, 255)),
                    (self.frMagenta, self.ilMagenta, (255, 0, 255)),
                    (self.frYellow, self.ilYellow, (255, 255, 0)),
                    (self.frCyanLight, self.ilCyanLight, (255, 190, 255)),
                    (self.frMagentaLight, self.ilMagentaLight, (190, 255, 255))):
                if il[i] != -1:
                    # ink level valid
                    c = QColor(col[0], col[1], col[2])
                    fr.setPaletteBackgroundColor(c)
                    pb.setPaletteBackgroundColor(bg)
                    p = pb.palette()
                    p.setColor(QColorGroup.Highlight, c)
                    if pb == self.ilBlack:
                        p.setColor(QColorGroup.HighlightedText, QColor(255, 255, 255))
                    pb.setPalette(p)
                    pb.setProgress(il[i])
                else:
                    # ink level not valid
                    fr.setPaletteBackgroundColor(bg)
                    pb.setPaletteBackgroundColor(bg)
                    pb.reset()
                i = i + 1
        else:
            # reset ink levels
            for fr, pb in (
                (self.frBlack, self.ilBlack),
                (self.frCyan, self.ilCyan),
                (self.frMagenta, self.ilMagenta),
                (self.frYellow, self.ilYellow),
                (self.frCyanLight, self.ilCyanLight),
                (self.frMagentaLight, self.ilMagentaLight)):
                fr.setPaletteBackgroundColor(bg)
                pb.setPaletteBackgroundColor(bg)
                pb.reset()
            self.errMessage(err)


    def errMessage(self, err):
        dev = self.cbDevice.currentText()
        QMessageBox.warning(self, self.trUtf8("EpsUtil Problem"),
            self.trUtf8(
    """<b>%1</b> doesn't appear to be a valid printer device!<br><br><b>escputil:</b><br>%2.<br>"""
            ).arg(dev).arg(err),
            QMessageBox.Abort | QMessageBox.Escape)

